Ext.namespace('fibroline.views.example', 'fibroline.views.contact');

Fibroline = Ext.regApplication({
	defaultTarget  : "viewport",
	name           : "fibroline",
	useLoadMask    : true,
    appModel       : null,
    appStore       : null,
    seguimientoMgr : null,
    activeInteraction : null, 
    
    /* write here the controllers and its default action */
    controllers   : [
                     { 
                    	 tab_order  : 0,
                    	 controller : 'fibroline.controllers.homeController',
                    	 action     : "index",
                    	 text       : "Home",
                    	 iconCls    : "home",
                    	 isActive   : true
                     }, {
                    	 tab_order  : 1,
                    	 action     : "index",
                    	 controller : 'fibroline.controllers.contactController',
                    	 iconCls    : "event_complete",
                    	 text       : "Tareas",
                    	 isActive   : false
         		  	}, {
         		  		tab_order  : 2,
         		  		action     : "index",
         		  		text       : "Lecciones",
         		  		iconCls    : "compose2",
         		  		controller : 'fibroline.controllers.leccionesController',
                   	    isActive   : false
         		  	}, {
         		  		tab_order  : 3,
         		  		action     : "index",
         		  		text       : "Estadisticas",
         		  		iconCls    : "chart2",
         		  		controller : 'fibroline.controllers.statisticsController',
                   	    isActive   : false
         		  	},{
         		  		tab_order  : 4,
         		  		action     : "index",
         		  		text       : "Preferencias",
         		  		iconCls    : "settings6",
         		  		controller : 'fibroline.controllers.preferencesController',
                   	    isActive   : false
         		  	},
         		  	{
         		  		tab_order  : 5,
         		  		action     : "activeSeguimiento",
         		  		text       : "Seguimiento",
         		  		iconCls    : "heart_circle",
         		  		controller : 'fibroline.controllers.exampleController',
                   	    isActive   : false
         		  	}
                     
	],
	
	launch: function() {
		Ext.Dispatcher.on('dispatch', this.dispath, this);

		this.seguimientoMgr = new fibroline.seguimiento.SeguimientoMgr({application : this});
		this.appStore = Ext.StoreMgr.get('fibroline.stores.ApplicationStore');
		this.appModel = Ext.ModelMgr.getModel('fibroline.models.ApplicationModel');

		// remove trhis line in production!!!!
		//this.appStore.proxy.getStorageObject().clear()
		
		this.viewport = new fibroline.views.Viewport({
			application : this,
			controllers : this.controllers
		});
		this.viewport.tabBar.on({
			change: this.changeTabBar,
			hide  : this.hideTabBar,
			scope : this
		});
		
		this.initInteractions();
		this.goTo(this.activeInteraction);
	},
	
	initInteractions: function() {
		this.appStore.load();
		if (this.appStore.getCount() == 0) {
			this.appStore.add(this.controllers);
			this.appStore.sync();
		}
		this.activeInteraction = this.appStore.findRecord('isActive', true);
	},
	
	updateActiveInteraction: function(nextController, nextAction) {
		var result = false;
		if (this.activeInteraction.get('controller') != nextController) {
			this.activeInteraction.set('isActive', false);
			this.activeInteraction = this.appStore.findRecord("controller", nextController);
			this.activeInteraction.set('isActive', true);
			result = true;
		}
		if (this.activeInteraction.get('action') != nextAction) {
			result = true;
			this.activeInteraction.set('action', nextAction);
		}
		if (result) {
			this.activeInteraction.setData(this.activeInteraction.getController().getData());
			this.appStore.sync();
		}
		return result;
	},
	
	dispath: function(interaction) {
		if (!Ext.isDefined(interaction.controller) || !Ext.isDefined(interaction.action)
				|| Ext.isEmpty(this.appStore.findRecord("controller", interaction.controller.id))) {
			return false;
		}
		
		var tabs           = this.viewport.tabBar.items;
		var oldTab         = tabs.getAt(this.activeInteraction.get("tab_order"));
		var newTab         = null;
		
		this.updateActiveInteraction(interaction.controller.id, interaction.action);
			
		newTab = tabs.getAt(this.activeInteraction.get("tab_order"));
			
		if (newTab != oldTab)
			oldTab.removeCls(oldTab.activeCls);
		newTab.addCls(newTab.activeCls);		
		return true;
	}, 
	
	 /**
	   * @public
	   * Used internally to set the view as active
	   * @param {Ext.Container} view to activate
	   */
	setCurrentView: function(controller, view, animation) {
		this.viewport.setActiveItem(view, animation);
	},
	
	/**
	 * Hide the tabBar.  Due the hide is performed by a transition, a callback and his scope will be
	 * passed in order to perform its after the hide transition
	 * @param {function} the callback without parameters
	 * @param {object} scope the "this" object inside the callback
	 */
	hideBar: function(fnAfterHide, scope) {
		this.fnAfterHide      = fnAfterHide;
		this.fnAfterHideScope = scope;
			
		this.viewport.tabBar.hide({type:'slide', direction:'up'})
	},
	
	showBar: function() {
		this.viewport.tabBar.show({type:'slide', direction:'up'})
	},
	
	hideTabBar: function(tabbar) {
		if (!Ext.isEmpty(this.fnAfterHide)) {
			this.fnAfterHide.apply(this.fnAfterHideScope);
		}
		this.fnAfterHide = this.fnAfterHideScope = null;
	},
	
	changeTabBar: function( tabbar, tab, card ) {		
		if (!Ext.isEmpty(tab.controller)) {
			var interaction = this.appStore.findRecord("controller", tab.controller);
			
			if (Ext.isEmpty(interaction)) {
				Ext.dispatch({
					controller: tab.controller,
					action: tab.action
				});
			} else {
				this.goTo(interaction);
			}
		}
	},
	
	/*
	 * go to an interaction setting the data
	 */
	goTo: function (interaction) {
		if (!Ext.isEmpty(interaction.get('data'))) {
			var controller = interaction.getController();
			controller.setData(interaction.getData());
		}
		Ext.dispatch({
			controller: interaction.get('controller'),
			action    : interaction.get('action')
		});
	},
	
	 /**
     * Devuelve el controlador del SeguimientoDiurno ((See {@link fibroline.seguimiento.Seguimiento}).
     * La idea es obtener dicho controlador para gestionar las historias de usuario relacionadas con el seguimiento diurno
     */
	getSeguimiento: function() {
		return this.seguimientoMgr;
	},
	
	/**
	 * read the active interaction: a instace of ApplicationModel 
	 * used by the Seguimiento to get the current state before start
	 */
	getActiveInteraction: function() {
		if (Ext.isDefined(this.activeInteraction))
			return this.activeInteraction.data;
		else
			return null;
	},
	
	/**
	 * get the preferences model instance
	 */
	getPreferences: function() {
		var controller = Ext.ControllerManager.get('fibroline.controllers.preferencesController');
		return controller.getPreferences();
	}

});
