/*
 * regController: crea un controlador y lo deja en Ext.ControllerManager con un id.
 * No es lazy, dado que lo crea.
*/
Ext.regController('fibroline.controllers.mesuresIniciController', {
	MSG: ['En este momento ¿cuánto dolor sientes? siendo 0 nada de dolor y 10 el máximo dolor imaginable',
	      'Estas caras expresan cuánto dolor puede sentir una persona. La cara del extremo izquierdo no expresa dolor, es decir, no siente ningún dolor. Las caras expresan más y más dolor hasta llegar a la del extremo derecho que tiene muchísimo dolor.  Señala la cara que mejor exprese el dolor que tienes ahora.'],
	
	indexView   : null,
	mesuresType : null,
	
	storeMesuresLocal : Ext.StoreMgr.get('fibroline.store.MesuresModel'),
	loadedStore : false,
	
	init: function() {		
		// insertamos la interaccion del index en el seguimiento diurno
		// El seguimiento diurno lo gestiona un objeto del tipo fibroline.seguimiento.Seguimiento
		// dicho objeto lo obtenemos de la aplicacion con la llamada: this.application.getSeguimientoDiurno()
		// dicha llamada nos devuelve un objeto del tipo Seguimiento.
		// Cuando el seguimiento diurno se dispare, entonces, en algún momento llamará a la interacción
		// index
		// A partir de ese momento el flujo y la totalidad de la pantalla la controla este controlador
		// y para salir del flujo, en la última ventana se debe llamar a una función que avise
		// al objeto seguimiento que éste ha recuperado el flujo
		this.application.on({
			launch: function(app) {
				this.preferences = this.application.getPreferences();
				this.mesuresType = this.preferences.get('escala_dolor');
				
				this.seguimiento = app.getSeguimiento();
				this.seguimiento.addInteractionDiurno({controller: this.id, action:"index"});				
			},
			scope: this
		})
		
	},
	
	index: function() {
		if (!this.loadedStore) {
			this.storeMesuresLocal.load({
				scope   : this,
	    		callback: function(records, operation, success) {
	    			if (success) {
	    				this.loadedStore = true;
	    				this.afterLoad();
					}
	    		}
			});
		} else {
			this.afterLoad();
		}
	},
	
	afterLoad: function () {
		var view = this.render ({
			xtype:'fibroline.views.mesures.MesuresInici',
			data: {msg: this.MSG[this.mesuresType]}
		});
		view.getNextButton().on({
			tap: function() {
				if (this.mesuresType == 0) {
					this.indexNumeric();
				} else {
					this.indexImatges();
				}
				
			},
			scope: this
		});
		this.application.setCurrentView(this, view);		
	}, 
	
/*
 * guardarMesura: deixa el valor de la mesura al proxy del model 'fibroline.models.MesuresModel'
 * 
 */
	guardarMesura: function(value) {
		var scale       = this.mesuresType,
			date		= new Date();
		
		//get a reference to the User model class
		var model = Ext.ModelMgr.create( {
				data		  : date,
				escala_dolor  : scale,
		    	valor   	  : value
		    }, 'fibroline.models.MesuresModel');
		    
		this.storeMesuresLocal.add(model);
		this.storeMesuresLocal.sync();
	},
	
	
/*
 * indexNumeric: Funcio que tracta la vista numérica i guarda la mesura presa
 */	
	indexNumeric: function() {
		if (Ext.isEmpty(this.indexView)) {
			this.indexView = this.render ({
				xtype:'fibroline.views.mesures.MesuresCarouselNumeric',
				listeners: {
				}
			});
			this.indexView.okButton.setHandler( function () {
				Ext.Msg.confirm("Confirmacion", "¿Seguro?", function (btn) {
					if (btn ==="yes") {
						var value = this.indexView.getValue();
						this.guardarMesura(value);
						this.seguimiento.next();
					}
				}, this);
			}, this);
		}
		this.indexView.init();
		this.application.setCurrentView(this, this.indexView, 'fade');
	},
	
	
/*
 * indexImatges: Funcio que tracta la vista d'imatges i guarda la mesura presa
 */	

	indexImatges: function() {
		if (Ext.isEmpty(this.indexView)) {
			this.indexView = this.render ({
				xtype:'fibroline.views.mesures.MesuresCarouselImatges',
				listeners: {
				}
			});
			this.indexView.okButton.setHandler( function () {
				Ext.Msg.confirm("Confirmacion", "¿Seguro?", function (btn) {
					if (btn ==="yes") {
						var value = this.indexView.getValue();
						this.guardarMesura(value);
						this.seguimiento.next();
					}
				}, this);
			}, this);
		}
		this.indexView.init();
		this.application.setCurrentView(this, this.indexView, 'fade');
	}
	
	});

fibroline.controllers.mesuresIniciController = Ext.ControllerManager.get('fibroline.controllers.mesuresIniciController');