Ext.namespace('fibroline.seguimiento');  

/**
 * @class fibroline.seguimiento.SeguimientoMgr
 * @extends Object
 * @author <a href="mailto:roman.roset@urv.cat">Roman Roset Mayals</a>
 * <p>Seguimiento es el objeto que gestiona las llamadas a las historias de usuario del seguimiento.</p>
 *
 * El seguimiento está compuesto de controladores.  Cada controlador que quiere aparecer en el seguimiento debe definir lo siguiente:
 * (Mirar CallerExample1 o CallerExample2) En la funcion init del controlador, se escucha el evento 'lanch2 de la aplicación
 * y una vez la aplicación se haya lanzado entonces obtenemos el seguimiento con aa.getSeguimiento, y añadimos la accion
 * <pre><code>
 * init: function() {		
		// insertamos la interaccion del index en el seguimiento diurno
		// El seguimiento diurno lo gestiona un objeto del tipo fibroline.seguimiento.Seguimiento
		// dicho objeto lo obtenemos de la aplicacion con la llamada: this.application.getSeguimientoDiurno()
		// dicha llamada nos devuelve un objeto del tipo Seguimiento.
		// Cuando el seguimiento diurno se dispare, entonces, en algún momento llamará a la interacción
		// index
		// A partir de ese momento el flujo y la totalidad de la pantalla la controla este controlador
		// y para salir del flujo, en la última ventana se debe llamar a una función que avise
		// al objeto seguimiento que éste ha recuperado el flujo
		this.application.on({
			launch: function(app) {
				this.seguimiento = app.getSeguimiento();
				this.seguimiento.addInteractionDiurno({controller: this.id, action:"index"});				
			},
			scope: this
		})
		
	},</code></pre>
 * 
 * Para lanzar el seguimiento se llama a la función startDiurno o startNocturno. Podemos probar en la consola de JS lo siguiente:
 * <pre><code>
    fibroline.getSeguimiento().startDiurno() // para iniciar el seguimiento Diurno
 * 
 * Cada controlodor dentro del seguimiento controla el flujo de las pantallas de ese controlador.  Una vez se ha llegado a una pantalla, 
 * si queremos pasar al siguiente controlador suscrito en el seguimiento, la acción del controlador debe llamar a la funcion end del seguimiento
 * (Mirar CallerExample1 o CallerExample2) 
 * <pre><code>
 * finalView: function() {
		
		var view = this.render({
			xtype: 'fibroline.views.wizard.CallerExampleViewFinal',
			listeners: {
				deactive: function(view) {
					view.destroy();
				}
		  	 }
		});
		
		view.getConfirmButton().on({
			tap: function() {
				var msgBox = Ext.Msg.confirm("Confirmation", "Are you sure you want to do that?", this.confirm, this);
			},
			scope: this
		});
		
		this.application.setCurrentView(this, view, 'fade');
	},
	
	confirm: function(yesNoString) {
		console.log(yesNoString);
		this.seguimiento.next();    // ATENCION <-- ES aquí donde se llama al seguimiento para informar que ha finalizado
	} </code></pre>
 */

fibroline.seguimiento.SeguimientoMgr = Ext.extend(Object, {
	/**
	 * @private {Array} interaction Array of {@link Ext.Interaction Ext.Interaction}. This works as heap, (FILO).
     * To add a iteration use the add method.
     */
	interactionsDiurno: null,
	
	/**
	 * @private {Array} interaction Array of {@link Ext.Interaction Ext.Interaction}. This works as heap, (FILO).
     * To add a iteration use the add method.
     */
	interactionsNocturno: null,
	
	/**
	 * @private {Integer} index of the current interaction. Only one is necessary (day or night)
     */
	currentInteraction: null, 
	
	/**
	 * @private {boolean} true if the wizard is displaying.
     */
	isActiveNocturno: null,
	
	/**
	 * @private {boolean} true if the wizard is displaying.
     */
	isActiveDiurno: null,
	
	// @private
	application       : null,
	//@private
	interactionModel  : null,
	//@private
	beforeInteraction : null,
	
	//@private: isActivateNocturno?
	isActiveNocturno: false,
	//@private: isActivateDiurno?
	isActiveDiurno: false,
	
	
	/*
	 * @private: initialization of the state stack
	 */
	initInteractions: function() {
		//get a reference to the User model class
		beforeInteraction = Ext.ModelMgr.getModel('fibroline.seguimiento.SeguimientoModel');
		
		//Uses the configured RestProxy to make a GET request to /users/123
		beforeInteraction.load(1, {
			failure: function(record, operation) {
		        this.beforeInteraction = null;
		    },
		    scope: this
		});
	},
	
	constructor: function(config) {
        var me = this;
        Ext.apply(me, config);
        
        me.interactionsDiurno   = [];
        me.interactionsNocturno = [];
        me.currentInteraction   = 0;
        me.isActiveNocturno     = false;
        me.isActiveDiurno       = false;
        //me.initInteractions();
    },
    
    /**
	 * Return the number of iterations in the wizard
     * @return {number} the number of iterations (actions) in the Wizard
	 */
	getNumControllersDiurno: function () {
		return this.interactionsDiurno.length
	},
	
	/**
	 * Return the number of iterations in the wizard
     * @return {number} the number of iterations (actions) in the Wizard
	 */
	getNumControllersNocturno: function () {
		return this.interactionsNocturno.length
	},
    
    /**
     * <p>Inserta una interaccion en el seguimiento diurno. La interaccion es un objeto del tipo {@link Ext.Interaction}.</p>
     * @param {Ext.Interaction} interaction The interaction to add in the wizard.
     * @return {number} order The order that has the interaction to be displayed in the wizard or null
     * if the interaction is not valid.
     */
    addInteractionDiurno: function (interaction) {
    	if (Ext.isEmpty(interaction) || Ext.isEmpty(interaction.controller) || Ext.isEmpty(interaction.action))
			return null;
		return this.interactionsDiurno.push(interaction) + 1;
    },
    
    /**
     * <p>Inserta una interaccion en el seguimiento nocturno. La interaccion es un objeto del tipo {@link Ext.Interaction}.</p>
     * @param {Ext.Interaction} interaction The interaction to add in the wizard.
     * @return {number} order The order that has the interaction to be displayed in the wizard or null
     * if the interaction is not valid.
     */
    addInteractionNocturno: function (interaction) {
    	if (Ext.isEmpty(interaction) || Ext.isEmpty(interaction.controller) || Ext.isEmpty(interaction.action))
			return null;
		return this.interactionsNocturno.push(interaction) + 1;
    },
    
    /**
     * <p> Con esta funcion empieza el seguimiento diurno
     */
    startDiurno: function() {
    	this.isActivateDiurno  = true;
    	this.beforeInteraction = Ext.ModelMgr.create(this.application.getActiveInteraction(), 'fibroline.seguimiento.SeguimientoModel');
    	this.beforeInteraction.save();
    	
    	
    	this.currentInteraction = 0;
        this.isActiveDiurno = true;
        this.application.hideBar(this.next, this);
    },
    
    /**
     * <p> Con esta funcion empieza el seguimiento nocturno
     */
    startNocturno: function() {
    	this.isActivateNocturno  = true;
    	this.beforeInteraction = Ext.ModelMgr.create(this.application.getActiveInteraction(), 'fibroline.seguimiento.SeguimientoModel');
    	this.beforeInteraction.save();
    	
    	this.currentInteraction = 0;
    	this.isActiveNocturno = true;
    	this.application.hideBar(this.next, this);
    },
    
    /**
     * <p> call the next interaction in the queue </p>
     */
    next: function() {
    	var nextInteraction = null;
    	var finish = false;
    	
    	if (this.isActiveDiurno) {
    		finish = this.currentInteraction == this.interactionsDiurno.length;
    		if (!finish)
    			nextInteraction = this.interactionsDiurno[this.currentInteraction++];
    	} else if (this.isActiveNocturno) {
    		finish = this.currentInteraction == this.interactionsNocturno.length;
    		if (!finish)
    			nextInteraction = this.interactionsNocturno[this.currentInteraction++];
    	}
    	if (finish) {
    		this.application.showBar();
    		this.application.goTo(this.beforeInteraction);
    		this.beforeInteraction.getProxy().clear();
    		this.beforeInteraction.clearListeners();
    		this.beforeInteraction = null;
    	} else {
    		Ext.dispatch(nextInteraction);
    	}
    },
});
